
#include "inc.h"

#include "lib.h"
#include "iwram_arm.h"
#include "div_arm.h"

#include "main.h"


//Mode0
//SPR LN^

//SPR
//  0       : @
//  1 `  7 : gp
//  8 `120 : XvCg_uǗ

//s16 Œ菬_
//
//F876 5432 1765 4321
//xxxx xxxx xxxx xxxx
//|          |
//       _

//---------------------------------------------------------------------------
//lib.c
extern ST_TIMER   Timer;

//iwram_arm.c
extern ST_DOBULER Doubler[2];
extern u8         DoublerSideSw;
extern u8         NextVCnt;
extern u32        DummyU32;

//---------------------------------------------------------------------------
ST_MYSHIP MyShip;
ST_SHOT   Shot[SHOT_MAX_CNT];
ST_BEAM   Beam[BEAM_MAX_CNT];
ST_BULLET Bullet[BULLET_MAX_CNT];
u16       ShotCnt;
u16       BeamCnt;
u16       BulletCnt;

u8        State;						//is󋵃tO
//---------------------------------------------------------------------------
EWRAM_CODE int main()
{
	State = STATE_INIT;

	for(;;)
	{
		switch(State)
		{
			case STATE_INIT:  StateInit();  break;
			case STATE_RESET: StateReset(); break;
			case STATE_GAME:  StateGame();  break;

			default:
				DebugMessage((u8*)"Err: Main State");
				break;
		}
	}

	return 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	InitWS();
	InitKey();
	InitSprite();
	InitDoubler();
	InitIRQ();

	SetMode(MODE_0 | BG2_ENABLE | OBJ_ENABLE | OBJ_MAP_2D);
	State = STATE_RESET;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitWS()
{
	//J[hbW̕Wݒ
	REG_WSCNT = 0x4317;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitKey()
{
	KeyInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitSprite()
{
	SpriteSetPal16((u16*)&spr_chr_pal_bin);
	SpriteSetData ((u16*)&spr_chr_img_bin, SPR_CHR_IMG_SIZE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitDoubler()
{
	DoublerSideSw = 0;
	DoublerClear(0);	//ST_DOUBLER1 obt@܂
	DoublerClear(1);	//ST_DOUBLER2 obt@܂
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitIRQ()
{
	NextVCnt     = SCREEN_CY;

	REG_INTERUPT = (u32)IRQUserHandler;
	REG_DISPSTAT = DSTAT_USE_VCOUNT | DSTAT_VCOUNT_LINE(NextVCnt);
	REG_IE       = IRQ_BIT_VCOUNT   | IRQ_BIT_TIMER0;

	REG_TM0D     = 65536 - 2621;			// (((16*1024*1024) / 64) * 10) / 1000
	REG_TM0CNT   = TM_ENABLE | TM_FREQ_PER_64 | TM_USEIRQ;

	REG_IME      = IRQ_MASTER_ON;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateReset()
{
	MyShipInit();
	ShotInit();
	BeamInit();
	BulletInit();

	MyShipNew(INT2FIX(0), INT2FIX(60));

	State = STATE_GAME;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MyShipInit()
{
	MyShip.x         = 0;
	MyShip.y         = 0;
	MyShip.roll      = 0;
	MyShip.shotGauge = 0;
	MyShip.shotTime  = 0;
	MyShip.beamPower = 0;
	MyShip.beamTime  = 0;
	MyShip.sprNo     = 0;

	SpriteSetSize(MyShip.sprNo, SP_SIZE_16, SP_SQUARE, SP_COLOR_16);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ShotInit()
{
	s16 i;

	for(i=0; i<SHOT_MAX_CNT; i++)
	{
		Shot[i].x  = 0;
		Shot[i].y  = 0;
		Shot[i].vx = 0;
		Shot[i].vy = 0;
	}
	ShotCnt = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void BeamInit()
{
	s16 i;

	for(i=0; i<BEAM_MAX_CNT; i++)
	{
		Beam[i].y  = 0;
	}
	BeamCnt = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void BulletInit()
{
	s16 i;

	for(i=0; i<BULLET_MAX_CNT; i++)
	{
		Bullet[i].x     = 0;
		Bullet[i].y     = 0;
		Bullet[i].vx    = 0;
		Bullet[i].vy    = 0;
		Bullet[i].timer = 0;
		Bullet[i].pMove = NULL;
	}
	BulletCnt = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MyShipNew(s16 x, s16 y)
{
	MyShip.x     = x;
	MyShip.y     = y;
	MyShip.sprNo = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateGame()
{
	//WaitForVCOUNT
	while(REG_VCOUNT != 153){};
	while(REG_VCOUNT == 153){};

	DoublerChange();

	DrawMain();
	ProdMain();
	MoveMain();
	CalcMain();
}
//---------------------------------------------------------------------------
EWRAM_CODE void DrawMain()
{
	MyShipDraw();
}
//---------------------------------------------------------------------------
EWRAM_CODE void MoveMain()
{
	MyShipMove();
	ShotMove();
	BeamMove();
	BulletMove();
}
//---------------------------------------------------------------------------
EWRAM_CODE void ProdMain()
{
	static u16 Cnt = 0;

	Cnt += 5;
	BulletNew(INT2FIX(0), INT2FIX(0), Cnt, (void*)&BulletMoveDir);

return;

/*
	if( _UMod(Timer.clockTick, 100) != 0)
	{
		return;
	}

	s16 i;
	u8  dir;

	switch (Cnt++)
	{
	//_e
	case 0:
		dir = CommonGetAngle(MyShip.x, MyShip.y, 0, -40);
		BulletNew(INT2FIX(0), INT2FIX(-40), dir, (void*)&BulletMoveDir);
		break;

	//e
	case 1:
		for(i=0; i<256; i+=4)
		{
			BulletNew(INT2FIX(0), INT2FIX(0), i * 16, (void*)&BulletMoveDir);
		}
		break;

	//e
	case 3:
		BulletNew(INT2FIX(0), INT2FIX(-40), 64, (void*)&BulletMoveSplit);
		break;

	default:
		Cnt = 0;
		break;
	}
*/
}
//---------------------------------------------------------------------------
void CalcMain()
{
	BulletCalc();
}
//---------------------------------------------------------------------------
EWRAM_CODE void MyShipDraw()
{
	s16 x   = CommonToFieldX(MyShip.x) - 8;			//8́AMYSHIP_CX / 2
	s16 y   = CommonToFieldY(MyShip.y) - 8;
	s16 chr = ( _Div(MyShip.roll, 10) + 3 ) * 2;

	SpriteSetChr(MyShip.sprNo, chr);
	SpriteMove  (MyShip.sprNo, x, y);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MyShipMove()
{
	//ړx
	s16 vx = INT2FIX(1);
	s16 vy = INT2FIX(1);

	ST_MYSHIP* p = &MyShip;
	u16 Key      = CommonScanKey();

	//Vbgƃr[̔˂܂
	if(Key & KEY_A)
	{
		//p[ȂƂAVbg𔭎˂܂
		if(p->beamPower < MYSHIP_BEAM_MIN_POWER)
		{
			p->beamPower++;
			p->shotGauge = MYSHIP_SHOT_MAX_GAUGE;

//			if(p->shotTime == 0) MyShipShot();
		} 
		else
		{
			p->shotGauge = 0;
//			if(p->beamTime == 0) MyShipBeam();

			vx = FIX_MUL(vx, 32);		//0.5--> INT2FIX(0.5) --> 32
			vy = FIX_MUL(vy, 32);
		}
	}
	else
	{
		//Q[WcĂƂVbg𔭎˂܂
		if(p->shotGauge > 0)
		{
//			if(p->shotTime == 0) MyShipShot();
			p->shotGauge--;
		}

		MyShipDeleteBeam();
		p->beamPower = 0;
	}

	//Vbgƃr[̑ҋ@ԂXV
	p->shotTime = _UMod(p->shotTime + 1, MYSHIP_SHOT_MAX_TIME);
	p->beamTime = _UMod(p->beamTime + 1, MYSHIP_BEAM_MAX_TIME);

	//Eړƃ[
	if(Key & KEY_LEFT)
	{
		vx = -vx;
		if(p->roll > -MYSHIP_MAX_ROLL) p->roll -= MYSHIP_VROLL;
	}
	else if(Key & KEY_RIGHT)
	{
		if(p->roll <  MYSHIP_MAX_ROLL) p->roll += MYSHIP_VROLL;
	}
	else
	{
		vx = 0;
		if((p->roll == 0) == FALSE)
		{
			p->roll += (p->roll < 0) ? MYSHIP_VROLL : -MYSHIP_VROLL;
		}
	}

	//㉺ړ
	if(Key & KEY_UP)
	{
		vy = -vy;
	}
	else if( !(Key & KEY_DOWN) )
	{
		vy = 0;
	}

	//΂ߕ̑x
	if(vx != 0 && vy != 0)
	{
		vx = FIX_MUL(vx, 44);		//1/sqrt(2.0f) = 0.7f  --> INT2FIX(0.7) --> 44.8
		vy = FIX_MUL(vy, 44);
	}

	//ړ
	p->x += vx;
	p->y += vy;

	//ړ͈͂̐
	if(p->x < -MYSHIP_MAX_X)      p->x = -MYSHIP_MAX_X;
	else if (MYSHIP_MAX_X < p->x) p->x =  MYSHIP_MAX_X;

	if(p->y < -MYSHIP_MAX_Y)      p->y = -MYSHIP_MAX_Y;
	else if (MYSHIP_MAX_Y < p->y) p->y =  MYSHIP_MAX_Y;
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 CommonScanKey()
{
	if( KeyWaitTimer() == TRUE )
	{
		return KEY_NONE;
	}
	if( KeyRepeat() == TRUE )		//keyGet֐ĂԑOɕKsȂ
	{
		return KEY_NONE;
	}

	return KeyInput();
}
//---------------------------------------------------------------------------
EWRAM_CODE void MyShipShot()
{
	ShotNew(MyShip.x, MyShip.y, 182);
	ShotNew(MyShip.x, MyShip.y, 192);
	ShotNew(MyShip.x, MyShip.y, 202);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ShotNew(s16 x, s16 y, s16 dir)
{
	if(ShotCnt >= SHOT_MAX_CNT)
	{
		return;
	}

	Shot[ShotCnt].x  = x;
	Shot[ShotCnt].y  = y;
	Shot[ShotCnt].vx = 2 * _Cos(dir);
	Shot[ShotCnt].vy = 2 * _Sin(dir);

	ShotCnt++;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ShotMove()
{
//TODO _upɏCKv
/*
	s16 i;

	for(i=0; i<ShotCnt; i++)
	{
		Shot[i].x += Shot[i].vx;
		Shot[i].y += Shot[i].vy;

		if( CommonOut(Shot[i].x, Shot[i].y, 5) == TRUE )
		{
			Shot[i].x  = Shot[ShotCnt - 1].x;		//Shot[i] = Shot[ShotCnt]memcpyg
			Shot[i].y  = Shot[ShotCnt - 1].y;
			Shot[i].vx = Shot[ShotCnt - 1].vx;
			Shot[i].vy = Shot[ShotCnt - 1].vy;

			ShotCnt--;
			i--;
		}
	}
*/
}
//---------------------------------------------------------------------------
EWRAM_CODE void MyShipBeam()
{
	BeamNew(MyShip.y);
}
//---------------------------------------------------------------------------
EWRAM_CODE void BeamNew(s16 y)
{
	if(BeamCnt >= BEAM_MAX_CNT)
	{
		return;
	}

	Beam[BeamCnt].y = y;
	BeamCnt++;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MyShipDeleteBeam()
{
	BeamCnt = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void BeamMove()
{
//TODO _upɏCKv
/*
	s16 i;

	for(i=0; i<BeamCnt; i++)
	{
		Beam[i].y -= INT2FIX(6);

		if( CommonOut(MyShip.x, Beam[i].y, 8) == TRUE )
		{
			Beam[i] = Beam[BeamCnt - 1];
			BeamCnt--;
			i--;
		}
	}
*/
}
//---------------------------------------------------------------------------
EWRAM_CODE void BulletNew(s16 x, s16 y, s16 dir, void pMove(void*))
{
	if(BulletCnt >= BULLET_MAX_CNT)
	{
		return;
	}

	Bullet[BulletCnt].x     = x;
	Bullet[BulletCnt].y     = y;
	Bullet[BulletCnt].vx    = _Cos(dir) >> 3;
	Bullet[BulletCnt].vy    = _Sin(dir) >> 3;
	Bullet[BulletCnt].pMove = pMove;
	Bullet[BulletCnt].timer = 0;

	BulletCnt++;
}
//---------------------------------------------------------------------------
EWRAM_CODE u8 CommonGetAngle(s16 fx1, s16 fy1, s16 fx2, s16 fy2)
{
	s16 dx = FIX2INT(fx1) - fx2;
	s16 dy = FIX2INT(fy1) - fy2;

	return _Angle(dx, dy);
}
//---------------------------------------------------------------------------
EWRAM_CODE bool CommonOut(s16 fx, s16 fy, s16 fsize)
{
	s16 x    = FIX2INT(fx);
	s16 y    = FIX2INT(fy);
	s16 size = FIX2INT(fsize);

	return x<-120-size || 120+size<x || y<-80-size || 80+size<y;
}
